import java.util.*;
/** Demo the development of recursive methods */
public class D {
  
  /** Implicit precondition of all String parameters: they are not null. */
  
  /** = sum of  digits in the decimal representation of n.
    e.g. sum(0) = 0, sum(3) = 3
    sum(34) = 7. 
    sum(1356) = 15.     6  + sum of the digits in 135
    Precondition: n >= 0. */
  public static int sumDigs(int n) {
    if (n < 10) {
      return n;
    }
    // {10 <= n}
    // return n%10 + sum of digits of n/10
    return n%10 + sumDigs(n/10);
  }
  
  /** = the length of s --without using function s.length. */
  public static int len(String s) {
      if (s.equals("")) return 0;
      
      // s  is s[0] + s[1..]
      // return 1 + length of s[1..]
      return 1 + len(s.substring(1));
  }
  
  /** = number of 'e's in s */
  public static int countEm(String s)  {
      if (len(s) == 0) return 0;
      
      // s is s[0] + s[1..]
      if (s.charAt(0) == 'e') return 1 + countEm(s.substring(1));
      return countEm(s.substring(1));
  }
  
  /** = number of the digits in the decimal representation of n.
    e.g. numDigits(0) = 1, numDigits(3) = 1, numDigits(34) = 2. numDigits(1356) = 4.
    Precondition: n >= 0. */
  public static int numDigits(int n) {
      if (n < 10) return 1;
      
      // n  >= 10, so it has at least 2 digits
      return 1 + numDigits(n/10);
  }
  
  /** = s with adjacent duplicates removed.
    Example: for s = "abbcccdeaaa", the answer is "abcdea".*/
  public static String rem1(String s) {
      if (s.length() <= 1) return s;
      
      // s is s[0] + s[1] + s[2..]
      // if s[0] = s[1], the answer is s[1..] with adjacent duplicates removed
      if (s.charAt(0) == s.charAt(1)) return rem1(s.substring(1));
      
      // s[0] != s[1]  // the answer is s[0] + (s[1..] with adj dups removed)
      return s.charAt(0) + rem1(s.substring(1));
  }
  
  /** = s with every char duplicated. */
  public static String dup(String s) {
      if (s.length() == 0) return s;
      
      // s is s[0] + s[1..]
      // answer is s[0] + s[0] +(s[1//] with every char duplicated
    return "" + s.charAt(0) + s.charAt(0) + dup(s.substring(1));
  }
  
  /** = b^c.
   * Precondition: c >= 0. */
  public static double exp(double b, int c) {
      // properties of ^: b^0 = 1.
      //                  for c > 0, b^c =  b * b^(c-1)
      if (c == 0) return 1;
      // c > 1.  Use second property
      return b * exp(b, c-1);
  }
  
  /** = "s is a palindrome" --reads the same backward and forward. */
  public static boolean isPal(String s) {
      if (s.length() <= 1) return true;
      int n= s.length() -1;
      // s is s[0] + s[1..n-1] + s[n]
      return s.charAt(0) == s.charAt(n) &&
              isPal(s.substring(1, n));
  }
  

  /** = the reverse of s */
  public static String rev(String s) {
      if (s.length() <= 1) return s;
      int n= s.length() - 1;
      // s is s[0] + s[1..n-1] + s[n] so the answer is
      //      s[n] + (reverse of s[1..n-1]) + s[0]
   return s.charAt(n) + rev(s.substring(1, n)) + s.charAt(0);
  }
  
  
  
  
  
  /** = the permutations of s.
    e.g. the permutations of "abc" are 
    "abc", "acb", "bac", "bca", "cab", "cba"
    Precondition: the chars of s are all different.*/
  public static Set<String> perms(String s) {
      // We store the permutations in a HashSet. All you have to know
      // is that a HashSet maintains a set. But we cast the set up
      // abstract class Set so only set operations will be used on it.
      // Take a look at how this method is tested in the JUnit testing class/
      if (s.length() <= 1) {
          Set<String> ans= new HashSet<String>();
          ans.add(s);
          return ans;
      }
      
      // s is s[0] + s[1..]
      // Its permutations are all the permutations of s[1..]
      // but with s[0] inserted in each possible position of each
      // of the permutations of s[1..].
      Set<String> p= perms(s.substring(1));

      Set<String> ans= new HashSet<String>();
      // Add to ans all permutations resulting from
      // inserting s[0] in all possible places of
      // all strings in p
      for (String r : p) {
          for (int k= 0; k <= r.length(); k= k+1) {
              ans.add(r.substring(0,k) + s.charAt(0)  + r.substring(k));
          }
      }
      
      return ans;
  }
  
}